//based on Tutorials (https://learnopengl.com/#!Lighting/Multiple-lights) && https://learnopengl.com/#!Advanced-Lighting/Bloom by JoeyDeVries
#version 430 core

struct Material {
    sampler2D texture_diffuse1;
	sampler2D texture_diffuse2;
    sampler2D texture_specular1;
	sampler2D texture_specular2;
    float shininess;
};

uniform sampler2D shadowMap;

struct Default_Material{
	vec3 diffuseColor;
	vec3 specularColor;
	float shininess;
};

struct Light {
    vec3 position;
	vec3 direction;
    
    float constant;
    float linear;
    float quadratic;
	
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

in vec2 texCoords;
in vec3 normal;
in vec3 fragmentPosition;
in vec4 fragmentPositionLightSpace;

out vec4 color;
out vec4 bright;

//max nr of lights
#define MAX_NR_LIGHTS 100

uniform Material material;
uniform Default_Material default_material;
uniform bool useDefaultMaterial;
uniform bool useSpecular;
uniform bool luminanceOnly;
uniform vec3 viewPos;
uniform int lightsUsed;
uniform Light lights[MAX_NR_LIGHTS];

vec3 threshold = vec3(0.2126, 0.7152, 0.0722) * 2.0f;

float shadowIntensity(vec4 fragPosLightSpace, vec3 normal, vec3 lightDir)
{
    // perspective has to be divided (transformed to clip space), because divide is only done automatically when passed to gl_Position in vertex shader
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range (range of shadow map)
    projCoords = projCoords * 0.5 + 0.5;
	// if outside frustum return not in shadow
	if(projCoords.z > 1.0)
	{
		return 0.0f;
	}

    // depth of current fragment
    float currentDepth = projCoords.z;
    // angle dependent bias to counteract moire pattern (results in greater bias at an angle)
	float bias = max(0.0008 * (1.0 - dot(normal, lightDir)), 0.00008);  

	// shadow testing and filtering (PCF)
    float shadow = 0.0;
	vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
	int filterSz = 2;

	// check if filter neighbours are in shadow and sum all shadow values
	for(int x = -filterSz; x <= filterSz; ++x)
	{
		for(int y = -filterSz; y <= filterSz; ++y)
		{
			float closestDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; // get closest object from shadowMap at coordinates
			shadow += currentDepth - bias > closestDepth ? 1.0 : 0.0; // check if current depth is further -> filter fragment is in shadow -> sum up
		}    
	}

	// divide by number of filter samples
	shadow /= pow(2 * filterSz + 1, 2);

    return shadow;
}

void main()
{    
	if(luminanceOnly)
	{
		vec4 texSample = texture(material.texture_diffuse1, texCoords);
		color = vec4(texSample.rgb, 1.0f);
		float brightness = dot(color.rgb, threshold);
		bright = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	}
	else
	{
		// renormalize interpolated normal
		vec3 norm = normalize(normal);
		vec3 viewDir = normalize(viewPos - fragmentPosition);
   
		vec3 result = vec3(0.0, 0.0, 0.0);

		//for each light
		for(int i = 0; i < lightsUsed; i++)
		{
			vec3 lightDir;
			float attenuation = 1.0f; // no attenuation for directional lights (= 1.0)
			float shadow = 0.0f;

			// this is a directional light if it has a non zero direction component
			if(length(normalize(lights[i].direction)) > 0.5)
			{
				// do shadow only for directional light
				lightDir = normalize(-lights[i].direction);
				shadow = shadowIntensity(fragmentPositionLightSpace, norm, lightDir);
			}
			else // this is a point light, do normal lighting procedure (no shadows)
			{
				lightDir = normalize(lights[i].position - fragmentPosition);
				float dist = length(lightDir);
				attenuation /= (lights[i].constant + lights[i].linear * dist + lights[i].quadratic * (dist * dist));
			}
		
			// Textured Lighting
			if(!useDefaultMaterial)
			{

				// diffuse = diffuseColor * Intensity * cos? = kd * Il * n dot l
				result += vec3(texture(material.texture_diffuse1, texCoords)) * lights[i].diffuse * max(dot(norm, lightDir), 0.0);
				//result += vec3(default_material.diffuseColor) * lights[i].diffuse * max(dot(norm, lightDir), 0.0);


				if(useSpecular)
				{
					// specular = specColor * Intensity * cos(angle(r,v))^shininess = ks * I * (n dot h)^shininess
					vec3 viewDir = normalize(viewPos - fragmentPosition);
					vec3 h = normalize(lightDir + viewDir);
					result += vec3(texture(material.texture_specular1, texCoords)) * lights[i].specular * pow(max(dot(norm, h), 0.0), material.shininess);
				}

				//shadow
				result *= (1.0 - shadow);

				// ambient
				result += vec3(texture(material.texture_diffuse1, texCoords)) * lights[i].ambient;
			}

			// Default Material Lighting
			else
			{

				//diffuse
				result += default_material.diffuseColor * lights[i].diffuse * max(dot(norm, lightDir), 0.0);

				// specular = specColor * Intensity * cos(angle(r,v))^shininess = ks * I * (n dot h)^shininess
				vec3 viewDir = normalize(viewPos - fragmentPosition);
				vec3 h = normalize(lightDir + viewDir);
				result += default_material.specularColor * lights[i].specular * pow(max(dot(norm, h), 0.0), default_material.shininess);

				result *= (1.0 - shadow);

				// ambient
				result += default_material.diffuseColor * lights[i].ambient;
			}
			result *= attenuation;
		}

		color = vec4(result, 1.0);
		float brightness = dot(color.rgb, threshold);
		if(brightness > 1.0)
			bright = vec4(color.rgb, 1.0);
		else
			bright = vec4(0.0f, 0.0f, 0.0f, 1.0f);
	}
	
}